<?php
/**
 * Plugin
 *
 * Part of the integration
 *
 * @package  Okiano
 */

namespace Okiano;

defined( 'ABSPATH' ) || exit;

/**
 * Plugin
 */
class Plugin {
	const ALREADY_BOOTSTRAPED      = 1;
	const DEPENDENCIES_UNSATISFIED = 2;
	const DISMISS_POSTFIX          = '_dismissed';

	/**
	 * Bootstrapped
	 *
	 * @var bool
	 */
	private $bootstrapped = false;

	/**
	 * Btart
	 *
	 * @return void
	 */
	public function start() {
        add_action('admin_menu', array(&$this, 'ffm_menu_page'));
        
		register_activation_hook( OKN_WC_PLUGIN_FILE, array( $this, 'activate' ) );
		register_deactivation_hook( OKN_WC_PLUGIN_FILE, array( $this, 'deactivate' ) );

		add_action( 'plugins_loaded', array( $this, 'bootstrap' ) );

		add_filter( 'plugin_action_links_' . plugin_basename( OKN_WC_PLUGIN_FILE ), array( $this, 'plugin_action_links' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );

		if ( is_admin() ) {
			add_action( 'wp_ajax_' . OKN_WC_ACTION_DISMISS_NOTICE, array( $this, 'ajax_dismiss_notice' ) );
		}

		add_action( 'admin_print_styles', array( $this, 'add_okn_stylesheet' ) );

		self::finish_onboarding();
	}

	/**
	 * Add_stylesheet
	 *
	 * @return void
	 */
	public function add_okn_stylesheet() {
		wp_enqueue_style( 'okianoCSS', plugins_url( '../style.css', __FILE__ ), array(), '1.0' );
	}

	/**
	 * Bootstrap
	 *
	 * @return void
	 * @throws \Exception An excepion to be caught.
	 */
	public function bootstrap() {
		try {
			if ( $this->bootstrapped ) {
				throw new \Exception( __( 'Okiano Marketing WooCommerce plugin can only be called once', 'okiano-marketing' ), self::ALREADY_BOOTSTRAPED );
			}

			$this->check_dependencies();
			$this->run();

			$this->bootstrapped = true;
		} catch ( \Exception $e ) {
			if ( in_array( $e->getCode(), array( self::ALREADY_BOOTSTRAPED, self::DEPENDENCIES_UNSATISFIED ) ) ) {
				set_site_transient( OKN_WC_BOOTSTRAP_MESSAGE, $e->getMessage(), MONTH_IN_SECONDS );
			}

			add_action( 'admin_notices', array( $this, 'show_bootstrap_message' ) );
		}
	}

	/**
	 * Deactivate
	 *
	 * @return void
	 */
	public function deactivate() {
		wp_remote_post(
			OKN_WC_WOOOKIANO_BASE_URL . 'woocommerce/uninstall',
			array(
				'method'      => 'POST',
				'timeout'     => 45,
				'redirection' => 5,
				'httpversion' => '1.0',
				'blocking'    => false,
				'headers'     => array(),
				'body'        => array(
					'user'      => wp_get_current_user()->data,
					'store_url' => get_site_url(),
				),
				'cookies'     => array(),
			)
		);
	}

	/**
	 * Activate
	 *
	 * @return void
	 */
	public function activate() {
		update_option( OKN_WC_WELCOME_MESSAGE . self::DISMISS_POSTFIX, false );

		wp_remote_post(
			OKN_WC_WOOOKIANO_BASE_URL . 'woocommerce/install',
			array(
				'method'      => 'POST',
				'timeout'     => 45,
				'redirection' => 5,
				'httpversion' => '1.0',
				'blocking'    => false,
				'headers'     => array(),
				'body'        => array(
					'user'      => wp_get_current_user()->data,
					'store_url' => get_site_url(),
				),
				'cookies'     => array(),
			)
		);

		$has_account_id = true;
		$saved_settings = Helper::get_plugin_options();

		if ( empty( $saved_settings ) ) {
			update_option(
				Helper::get_option_key(),
				array(
					'account_id' => '',
				)
			);

			$has_account_id = false;
		}

		if ( ! $has_account_id ) {
			set_site_transient( OKN_WC_WELCOME_MESSAGE, true, MONTH_IN_SECONDS );
		}
    }

    /* Menu Page */
    public function ffm_menu_page()
    {
        add_menu_page(
        __('Okiano Ads', 'okiano-marketing'),
        __('Okiano Ads', 'okiano-marketing'),
        'manage_options',
        'wc-settings&tab=integration&section=okn_wcintegration',
        array(&$this, 'ffm_setup_callback'),
        plugins_url('../assets/icon-button.jpg', __FILE__)
        );
                    
    }

    public function ffm_setup_callback() {
    }

	/**
	 * Plugin_action_links
	 *
	 * @param  mixed $links a required parameter.
	 * @return mixed
	 */
	public function plugin_action_links( $links ) {
		$plugin_links = array();

		if ( function_exists( 'wc' ) ) {
			$plugin_links[] = '<a href="' . esc_url( Helper::get_plugin_page() ) . '">' . __( 'Dashboard', 'okiano-marketing' ) . '</a>';
		}

		$plugin_links[] = '<a href="' . OKN_WC_SUPPORT_LINK . '">' . __( 'Support', 'okiano-marketing' ) . '</a>';

		return array_merge( $plugin_links, $links );
	}

	/**
	 * Show_bootstrap_message
	 *
	 * @return void
	 */
	public function show_bootstrap_message() {
		$message = get_site_transient( OKN_WC_BOOTSTRAP_MESSAGE );

		if ( ! empty( $message )
			&& false === get_option( OKN_WC_BOOTSTRAP_MESSAGE . self::DISMISS_POSTFIX, false )
			&& current_user_can( 'manage_options' ) ) {

			Message::show_warning( $message, 'bsmessage' );
		}
	}

	/**
	 * Show_onboarding_message
	 *
	 * @return void
	 */
	public function show_onboarding_message() {
		$settings = Helper::get_plugin_options();

		if ( false !== get_site_transient( OKN_WC_WELCOME_MESSAGE, false )
			&& false === get_option( OKN_WC_WELCOME_MESSAGE . self::DISMISS_POSTFIX, false )
			&& current_user_can( 'manage_options' ) && ! Helper::is_plugin_page()
			&& empty( $settings['account_id'] ) ) {

			Message::show_info( Helper::get_onboarding_message(), 'welcome' );
		}
	}

	/**
	 * Woocommerce_integrations
	 *
	 * @param  mixed $integrations a required parameter.
	 * @return mixed
	 */
	public function woocommerce_integrations( $integrations ) {
		$integrations[] = '\\Okiano\\WC_Integration';

		return $integrations;
	}

	/**
	 * Wp_head
	 *
	 * @return void
	 */
	public function wp_head() {
		// Inject Google verification token.

		// Inject tracking script.
	}

	/**
	 * Rest_api_init
	 *
	 * @return void
	 */
	public function rest_api_init() {
		( new REST_Categories_Controller() )->register_routes();
	}

	/**
	 * Admin_init
	 *
	 * @return void
	 */
	public function admin_init() {
		Helper::check_redirect_for_wc_auth();
	}

	/**
	 * Admin_enqueue_scripts
	 *
	 * @return void
	 */
	public function admin_enqueue_scripts() {
		wp_enqueue_script( 'okn-admin-script', OKN_WC_PLUGIN_URL . 'admin-script.js', array( 'jquery' ), '1.0' );

		wp_localize_script(
			'okn-admin-script',
			'dimissibleNotice',
			array(
				'action'         => OKN_WC_ACTION_DISMISS_NOTICE,
				'nonce'          => wp_create_nonce( OKN_WC_ACTION_DISMISS_NOTICE ),
				'confirmMessage' => esc_html__( 'This will dismiss the message permanently. Are you sure?', 'okiano-marketing' ),
			)
		);
	}

	/**
	 * Clear_data
	 *
	 * @return void
	 */
	public function clear_data() {
		$url = Helper::get_plugin_page();

		Helper::clear_options();

		if ( $url ) {
			wp_safe_redirect( $url );
			exit;
		}
	}

	/**
	 * Save_account
	 *
	 * @return void
	 */
	public function save_account() {
		$url = OKN_WC_WOOOKIANO_BASE_URL . 'store-details';

		if ( isset( $_GET['user_id'] ) ) {
			Helper::save_account_info(
				sanitize_text_field( wp_unslash( $_GET['user_id'] ) )
			);
		}

		if ( $url ) {
			wp_redirect( $url );
			exit;
		}
	}

	/**
	 * Finish_onboarding
	 *
	 * @return mixed
	 */
	public function finish_onboarding() {
		$options = get_option( Helper::get_option_key() );

		if ( isset( $options['account_id'] ) ) {
			if ( isset( $options['finish_onboarding'] ) && true !== $options['finish_onboarding'] ) {
				$req = wp_remote_get( OKN_WC_WOOOKIANO_BASE_URL . 'woocommerce/redirect?redir=false&uid=' . $options['account_id'] );

				if ( is_wp_error( $req ) ) {
					return false;
				}

				$res  = wp_remote_retrieve_body( $req );
				$data = json_decode( $res );

				if ( true === $data->completed ) {

					Helper::save_finish_onboarding();

					wp_safe_redirect( Helper::get_plugin_page() );
					exit;
				}
			}
		}
	}

	/**
	 * Ajax_dismiss_notice
	 *
	 * @return void
	 */
	public function ajax_dismiss_notice() {
		check_ajax_referer( OKN_WC_ACTION_DISMISS_NOTICE );

		if ( isset( $_POST['message_name'] ) ) {
			$message_name = sanitize_key( $_POST['message_name'] );

			switch ( $message_name ) {
				case 'welcome':
					update_option( OKN_WC_WELCOME_MESSAGE . self::DISMISS_POSTFIX, true );
					break;
				case 'bsmessage':
					update_option( OKN_WC_BOOTSTRAP_MESSAGE . self::DISMISS_POSTFIX, true );
					delete_site_transient( OKN_WC_BOOTSTRAP_MESSAGE );
					break;
			}
		}

		wp_die();
	}

	/**
	 * Check_dependencies
	 *
	 * @return void
	 * @throws \Exception An excepion to be caught.
	 */
	private function check_dependencies() {
		if ( ! function_exists( 'wc' ) ) {
			throw new \Exception( 'Google Ads for WooCommerce requires WooCommerce to be activated', self::DEPENDENCIES_UNSATISFIED );
		}

		$required_woo_version = '3.0';
		if ( version_compare( wc()->version, $required_woo_version, '<' ) ) {
			throw new \Exception( sprintf( 'Google Ads for WooCommerce requires WooCommerce version %s or greater', $required_woo_version ), self::DEPENDENCIES_UNSATISFIED );
		}
	}

	/**
	 * Run
	 *
	 * @return void
	 */
	private function run() {
		add_filter( 'woocommerce_integrations', array( $this, 'woocommerce_integrations' ) );
		add_action( 'wp_head', array( $this, 'wp_head' ) );
		add_action( 'rest_api_init', array( $this, 'rest_api_init' ) );
		add_action( 'admin_init', array( $this, 'admin_init' ) );
		add_action( 'admin_notices', array( $this, 'show_onboarding_message' ) );

		if ( is_admin() ) {
			add_action( 'admin_action_' . OKN_WC_ACTION_SAVE_ACCOUNT, array( $this, 'save_account' ) );
			add_action( 'admin_action_' . OKN_WC_ACTION_CLEAR_DATA, array( $this, 'clear_data' ) );
		}
	}
}
